#### R script for downloading/cleaning OBIS and GBIF data, and plotting occurrences on polygons to code presence/absence

##### Install required packages
# devtools::install_github("iobis/robis")
library(raster)
library(sp)
library(dismo)
library(maptools)
library(rgdal)
library(robis)
library(rgeos)
library(gpclib)

##### Guide to QC codes following Vandepitte et al. 2015:

# 2 taxon name matches WoRMS
# 3 taxon lower than family level
# 4 lat/long greater than zero
# 5 lat/long within possible boundaries
# 6 coordinates on the coastline or buffer
# 9 coordinates in expected geographic area given metadata
# 18 min depth <= max depth
# 19 sampling depth possible
# 1 all required OBIS fields completed
# 10 Basis of record is documented with a valid code
# 28 Outlier analysis: IQR distance to centroid


#### PART ONE OF PIPELINE: Download two datasets for each species in OBIS 
## One dataset is just routine cleaning
## Second is routine cleaning + outlier detection
## In preliminary analyses, these two datasets were compared, and we found that using routine cleaned data only still allowed several errors.
## Instead, we used rc data only in cases where a species did not have sufficient data to perform outlier analysis. Thus, data for majority of marine species has outlier analysis performed.

setwd("~/Desktop/dryad for fish paper Dec17/a) phylogeny and biogeographic coding")

names <- read.csv("perc_complete_list_aug11_2016.csv") ## this is a file of all 17,458 FishBase names. This was compiled manually from the FishBase website (see Appendix S1)
names <- names$Species


#### WARNING: this pipeline will generate thousands of output files, and may take several days to complete. User will need to set working directories for these files.

setwd("~/Desktop/dryad for fish paper Dec17/a) phylogeny and biogeographic coding/output")

# Script may not run through all names smoothly, like if the name is not recognized. User may need to start/finish in pieces, skipping problematic names.

x<- 1 #counter
for(i in names[x:length(names)]){ #cycle through each name
  print(x) #to keep time through the list of >17,000 species
  
  # Just routine cleaning
  data <- occurrence(i, fields = c("decimalLongitude", "decimalLatitude"), qc = c(2,3,4,5,6,9,18,19,1,10))
  
  write.csv(data, file=paste0(i, "_rc.csv")) #rc = routine cleaning
  
  # All qc codes, includes outlier analysis
  data <- occurrence(i, fields = c("decimalLongitude", "decimalLatitude"), qc = c(2,3,4,5,6,9,18,19,1,10,28)) 

  write.csv(data, file=paste0(i, "_all.csv")) 
  
  x<- x+1
  
} # end loop through names

## Now, all species with OBIS data will have two .csv files of lat/long data. If the species wasn't in OBIS, they will have two blank files instead.



#### PART TWO OF PIPELINE: Plot occurrence data on shape file polygons, code presence/absence 

##### Prepare by reading in QGIS files

setwd("~/Desktop/dryad for fish paper Dec17/a) phylogeny and biogeographic coding/QGIS_shapefiles")

# NOTE: Use readOGR() instead of readShapeSpatial().
# NOTE: the REGION@data$id = rep(1, length(REGION@data$id)) command is necessary to set all attribute fields for the polygons to 1. 
WIP = readOGR("WIP.shp")
WIP@data$id = rep(1, length(WIP@data$id))
CIP = readOGR("CIP.shp") 
CIP@data$id = rep(1, length(CIP@data$id))
CP = readOGR("CP.shp")
CP@data$id = rep(1, length(CP@data$id))
EP = readOGR("EP.shp")
EP@data$id = rep(1, length(EP@data$id))
WA = readOGR("WA.shp")
WA@data$id = rep(1, length(WA@data$id))
EA = readOGR("EA.shp")
EA@data$id = rep(1, length(EA@data$id))
NC = readOGR("NC.shp")
NC@data$id = rep(1, length(NC@data$id))
SC = readOGR("SC.shp")
SC@data$id = rep(1, length(SC@data$id))


#### PLOT OBIS DATA ON POLYGONS

# Set working directory accordingly 
setwd("~/Desktop/fish files may 2017/obis_new/new_obis_dat/Users/Elizabeth/Desktop/fish files may 2017/obis_new/new_obis_data/new_outlier_aug28")

# Create a blank dataframe
distribution_obis = data.frame(matrix(data = NA, nrow = 17458, ncol = 10))
colnames(distribution_obis) = c("number", "scientific_name", "WA", "EA", "WIP", "CIP", "CP", "EP", "NC", "SC")
distribution_obis$number = c(1:length(distribution_obis$number))
distribution_obis$scientific_name<- as.character(distribution_obis$scientific_name)

area_list = c(WA,EA,WIP,CIP,CP,EP,NC,SC)

# The following code may take 10+ minutes to run
x<-1
for (i in names) {
  distribution_obis$scientific_name[x] = as.character(i)
  if (file.info(paste0(i, "_all.csv"))$size > 4) # not a blank file {
    print(i)
    occ = read.csv(paste0(i, "_all.csv"), header = TRUE, stringsAsFactors = FALSE)
    
    coordinates(occ) = ~decimalLongitude+decimalLatitude
    # Need to re-project points to match projection of polygons
    proj4string(occ) = CRS("+proj=longlat +datum=WGS84 +no_defs +ellps=WGS84 +towgs84=0,0,0")
    for (j in 1:8) {
      overlist = over(occ, area_list[[j]])
      if (length(which(overlist$id == 1)) > 0) {
        print(i)
        distribution_obis[x, j+2] = 1
        # 1 signifies occurrences in region
      }
      else {
        distribution_obis[x, j+2] = 0
        # 0 signifies no occurrence in region
      }
    }
  }
  x<-x+1
}


write.csv(distribution_obis, file = "distribution_obis_all.csv")


####### PART THREE OF PIPELINE: Repeat process with GBIF data (species not found in OBIS were then searched for in GBIF)


library(dismo)
library(biogeo)
library(rgbif)

names<-names[which(is.na(distribution_obis$WA))] #list of names with no data in OBIS

## Again, user will need to make a directory for GBIF output files 

y<-1
for(x in names[y:length(names)]){
  print(y)
  tem_result_gbif <- occ_search(scientificName = as.character(x), return = "data", hasCoordinate = TRUE,limit = 200000)
  y<-y+1
  if(is.data.frame(tem_result_gbif))
  {
    write.csv(tem_result_gbif, file=paste0(x, "_gbif.csv"))
  }}

################################################# CLEANING GBIF DATA 

library(speciesgeocodeR)
library(maptools)
data(countryref)
data(wrld_simpl)

x<-1
for(i in names[x:length(names)]){
  print(i)
  if(file.exists(paste0(i, "_gbif.csv"))){
    data <- read.csv(paste0(i, "_gbif.csv"))
    
    # format columns
    
    names(data)[which(names(data)=="species")]<- "identifier"
    names(data)[which(names(data)=="decimalLongitude")]<- "XCOOR"
    names(data)[which(names(data)=="decimalLatitude")]<- "YCOOR"
    names(data)[which(names(data)=="country")]<- "country_name"
    names(data)[which(names(data)=="countryCode")]<- "country"
    
    #simple clean
    
    d<- GeoClean(data, isna = TRUE, isnumeric = TRUE,
                 coordinatevalidity = TRUE, containszero = TRUE,zerozero = TRUE, zerozerothresh = 1,latequallong = TRUE, outp="cleaned") #will automatically remove bad points

    #these checks require a map and so are more complicated
    
    d <- GeoClean(d, capitalcoords = TRUE, countrycheck = TRUE,
                  polygons = wrld_simpl, GBIFhead = TRUE, countrycentroid = TRUE,
                  referencecountries = countryref, capthresh = 0.5,
                  contthresh = 0.5, outp = "cleaned")
    
    write.csv(d, file=paste0(i, "_gbif_countries.csv"))
    
  }} #end code


# Create a blank dataframe
distribution_gbif = data.frame(matrix(data = NA, nrow = length(names), ncol = 10))
colnames(distribution_gbif) = c("number.", "scientific_name", "WA", "EA", "WIP", "CIP", "CP", "EP", "NC", "SC")
distribution_gbif$number. = c(1:length(distribution_gbif$number.))
x<-1
distribution_gbif$scientific_name<- as.character(distribution_gbif$scientific_name)


# The following code will take ~5-10 minutes to run
for (i in names) {
  distribution_gbif$scientific_name[x] = as.character(i)
  if (file.exists(paste0(i, "_gbif_countries.csv")) & file.info(paste0(i, "_gbif_countries.csv"))$size > 42) {
    print(i)
    occ = read.csv(paste0(i, "_gbif_countries.csv"), header = TRUE, stringsAsFactors = FALSE)
    names(occ)[3]<-"decimalLongitude"
    names(occ)[4]<-"decimalLatitude"

    coordinates(occ) = ~decimalLongitude+decimalLatitude
    # Need to re-project points to match projection of polygons
    proj4string(occ) = CRS("+proj=longlat +datum=WGS84 +no_defs +ellps=WGS84 +towgs84=0,0,0")
    for (j in 1:8) {
      overlist = over(occ, area_list[[j]])
      if (length(which(overlist$id == 1)) > 0) {
        distribution_gbif[x, j+2] = 1
        # 1 signifies occurrences in region
      }
      else {
        distribution_gbif[x, j+2] = 0
        # 0 signifies no occurrence in region
      }
    }
  }
  x<-x+1
}

# Write csv of distribution_obis
write.csv(distribution_gbif, file = "distribution_gbif_countries.csv")

###### NOTE: These datasets were compiled manually to create the final version. Details are in the Extended Methods in Appendix S1. For example, we excluded marine area assignments from fishes that did not occur in "saltwater" habitats according to FishBase. 


