## This script calculates a weighted mean net diversification rate for each family/genus, and performs regression analyses.

## This script is for families. The script for genera is similar, and available by request.

# Input file with ages and richness of clades is included. 


####### First, calculate net diversification rates for each of 200 families.

data <- read.csv("net_div_regional_rich.csv")

library(laser) # As of December 2017 this package is defunct on CRAN; users will need to install it from archives

x <- 1
for(i in data$family){
  rate_data[x,1] <- i
  
  # stem 
  data[x,6] <- lambda.stem.ms01(data[x,3], data[x,5], eps=0)$r
  data[x,7] <- lambda.stem.ms01(data[x,3], data[x,5], eps=0.5)$r
  data[x,8] <- lambda.stem.ms01(data[x,3], data[x,5], eps=0.9)$r
  
  #crown
  data[x,9] <- lambda.crown.ms01(data[x,3], data[x,4], eps=0)$r
  data[x,10] <- lambda.crown.ms01(data[x,3], data[x,4], eps=0.5)$r
  data[x,11] <- lambda.crown.ms01(data[x,3], data[x,4], eps=0.9)$r
  
  x<-x+1
}


####### Second, calculate weighted net diversification rate for each region 

rate_data <- as.data.frame(matrix(NA,6,10))
colnames(rate_data) <- c("WA", "EA", "WI", "CIP", "CP", "EP", "NC", "SC", "FW", "FW_strict")
rownames(rate_data) <- names(data)[6:11]

#For each region, the net div is = to the sum of net div. of each family occuring in the area*number of species that family has in that area/total richness of the area.

## the columns for net.div stem correspond to 6, 7, 8 (0,0.5,0.9)
## the columns for net.div crown correspond to 9, 10, 11 (0,0.5,0.9)
## columns for areas correspond to: WA:12, EA:13, WIP:14, CIP:15, CP:16, EP:17, NC:18, SC:19, FW:20, FW_strict:21

#This script is run for one rate type at a time (of 6). Input these index codes manually:

rate<- 6 #choose the row (of 6 options) corresponding to the rate to be calculated
rate_row <- 1 #column corresponding to the rate of choice, input manually, 1:6

formula_list <- c() #the 10 values will go in here

areas <- c(12:21) #corresponds to columns of regional richness in data 

for(area in areas){
  print(area)
fams <- data[which(data[,area] > 0) , ] #change to area of interest
formula <- c()
for(i in 1:length(fams$family)){
  value <- fams[i, rate]*fams[i,area] #in order of column of net div * column of area rich.
  formula <- c(formula, value)
}
formula_list<- c(formula_list, sum(na.omit(formula)))
}

#formula list is the sum of the weighted net rates for all 10 regions

#richness values below are only for included clades (i.e. for stem rates, all clades sampled in the phylogeny. For crown rates, only clades with a crown age sampled in the phylogeny). This is inputted manually (see Table S7 in ESM)

rate_data[rate_row, 1] <- formula_list[1]/1711
rate_data[rate_row, 2] <- formula_list[2]/1118
rate_data[rate_row, 3] <- formula_list[3]/2550
rate_data[rate_row, 4] <- formula_list[4]/5364
rate_data[rate_row, 5] <- formula_list[5]/3453
rate_data[rate_row, 6] <- formula_list[6]/1486
rate_data[rate_row, 7] <- formula_list[7]/1677
rate_data[rate_row, 8] <- formula_list[8]/857
rate_data[rate_row, 9] <- formula_list[9]/6503
rate_data[rate_row, 10] <- formula_list[10]/4967
  
rate_data

#write.csv(rate_data, file="weighted_mean_rates_oct20.csv") #when finished, write to file


#################### Finally, perform regression analyses of mean weighted rates and richness

rate_data <- read.csv("weighted_mean_rates_oct20.csv")
rate_data$log_regrich <- log10(rate_data$reg_rich)

################################################################ STEM AGES

### wide definition of FW
rate_data <- rate_data[-10,] #remove fw strict

model_0 <- lm(log_regrich~netdiv_stem_0, data=rate_data)
model_5 <- lm(log_regrich~netdiv_stem_5, data=rate_data)
model_9 <- lm(log_regrich~netdiv_stem_9, data=rate_data)

capture.output(summary(model_0), file="netdiv_families_e0_stem.txt")
capture.output(summary(model_5), file="netdiv_families_e5_stem.txt")
capture.output(summary(model_9), file="netdiv_families_e9_stem.txt")

### strict def FW

rate_data <- read.csv("weighted_mean_rates_oct20.csv")
rate_data$log_regrich <- log10(rate_data$reg_rich)

rate_data <- rate_data[-9,] # remove wide def. of freshwater

model_0 <- lm(log_regrich~netdiv_stem_0, data=rate_data)
model_5 <- lm(log_regrich~netdiv_stem_5, data=rate_data)
model_9 <- lm(log_regrich~netdiv_stem_9, data=rate_data)

capture.output(summary(model_0), file="netdiv_families_e0_stem_strictFW.txt")
capture.output(summary(model_5), file="netdiv_families_e5_stem_strictFW.txt")
capture.output(summary(model_9), file="netdiv_families_e9_stem_strictFW.txt")


################################################################ CROWN AGES

rate_data <- read.csv("weighted_mean_rates_oct20.csv")
rate_data$log_regrich <- log10(rate_data$reg_rich)

### wide def FW
rate_data <- rate_data[-10,] #rm fw strict

model_0 <- lm(log_regrich~netdiv_crown_0, data=rate_data)
model_5 <- lm(log_regrich~netdiv_crown_5, data=rate_data)
model_9 <- lm(log_regrich~netdiv_crown_9, data=rate_data)

capture.output(summary(model_0), file="netdiv_families_e0_crown.txt")
capture.output(summary(model_5), file="netdiv_families_e5_crown.txt")
capture.output(summary(model_9), file="netdiv_families_e9_crown.txt")

### strict def FW

rate_data <- read.csv("weighted_mean_rates_oct20.csv")
rate_data$log_regrich <- log10(rate_data$reg_rich)

rate_data <- rate_data[-9,] # rm FW

model_0 <- lm(log_regrich~netdiv_crown_0, data=rate_data)
model_5 <- lm(log_regrich~netdiv_crown_5, data=rate_data)
model_9 <- lm(log_regrich~netdiv_crown_9, data=rate_data)

capture.output(summary(model_0), file="netdiv_families_e0_crown_strictFW.txt")
capture.output(summary(model_5), file="netdiv_families_e5_crown_strictFW.txt")
capture.output(summary(model_9), file="netdiv_families_e9_crown_strictFW.txt")
